# -*- coding: utf-8 -*-
#COPYRIGHT 2018 ESRI
#
#TRADE SECRETS: ESRI PROPRIETARY AND CONFIDENTIAL
#Unpublished material - all rights reserved under the
#Copyright Laws of the United States.
#
#For additional information, contact:
#Environmental Systems Research Institute, Inc.
#Attn: Contracts Dept
#380 New York Street
#Redlands, California, USA 92373
#
#email: contracts@esri.com
import io
import os
import re

__all__ = ['PDFDocument', 'PDFDocumentOpen', 'PDFDocumentCreate']

RANGE_RE = re.compile("(\d+)(?:\s*-\s*(\d+))?")

def page_range_list(range_string):
    pages = []
    for page_start, page_end in RANGE_RE.findall(range_string):
        page1 = int(page_start)
        if page_end:
            page2 = int(page_end)
            pages.extend(range(page1, page2 + 1))
        else:
            pages.append(page1)
    return pages

class PDFDocument(object):
    """Allows for the management of PDF documents, including facilities for
       merging and deleting pages, setting document open behavior, and creating
       or changing document security settings."""
    _open = False
    _decrypt = None
    _decrypt_key = None
    _ID = None
    _password = None
    _owner_password = None

    def __init__(self, pdf_path, input_pdf_password=None, open_new=False):
        """PDFDocument(pdf_path)

           Creates an empty PDFDocument object in memory.

             pdf_path(String):
           A string that specifies the  path and file name for the resulting PDF
           file when the saveAndClose method is called."""
        import PyPDF2

        self._PDFPath = pdf_path
        if not open_new:
            (self._info,
             self._layout,
             self._mode,
             self._decrypt,
             self._decrypt_key,
             self._ID,
             pw_type,
             self._pages) = self._pagesAndInfoForPath(pdf_path,
                                                      input_pdf_password)
            self._128bit = True
            self._open = True
        else:
            if os.path.exists(pdf_path):
                raise IOError("{} already exists".format(pdf_path))
            self._info = PyPDF2.pdf.DocumentInformation()
            self._layout = PyPDF2.generic.TextStringObject('/SinglePage')
            self._mode = PyPDF2.generic.TextStringObject('/UseNone')
            self._pages = []
            self._128bit = True
            self._open = True

    @classmethod
    def _pagesAndInfoForPath(cls, pdf_path, input_pdf_password=None):
        import PyPDF2

        if isinstance(pdf_path, cls):
            return (pdf_path._info,
                    pdf_path._layout,
                    pdf_path._mode,
                    pdf_path._decrypt,
                    pdf_path._decrypt_key,
                    pdf_path._ID,
                    0,
                    pdf_path._pages)
        elif not os.path.isfile(pdf_path):
            raise IOError("{} not found".format(pdf_path))

        with open(pdf_path, 'rb') as in_stream:
            in_io = io.BytesIO(in_stream.read())

        reader = PyPDF2.PdfFileReader(in_io)
        pw_type = 0
        if input_pdf_password is not None:
            pw_type = reader.decrypt(input_pdf_password)

        info = reader.getDocumentInfo()

        encrypt_string = PyPDF2.generic.TextStringObject("/Encrypt")
        id_string = PyPDF2.generic.TextStringObject("/ID")

        encrypt = None
        encrypt_key = None
        encrypt_id = None

        if encrypt_string in reader.trailer:
            encrypt_id = reader.trailer[id_string].getObject()
            encrypt = reader.trailer[encrypt_string].getObject()
            encrypt_key = reader._decryption_key

        return (info,
                reader.getPageLayout() or '/NoLayout',
                reader.getPageMode() or '/UseNone',
                encrypt, encrypt_key, encrypt_id, pw_type,
                [reader.getPage(page_number)
                 for page_number in range(reader.getNumPages())])

    @classmethod
    def _pagesForPath(cls, pdf_path, input_pdf_password=None):
        return cls._pagesAndInfoForPath(pdf_path, input_pdf_password)[-1]

    def appendPages(self, pdf_path, input_pdf_password=None):
        """PDFDocument.appendPages(pdf_path, {input_pdf_password})

           Appends one PDF document to the end of another.

             pdf_path(String):
           A string that includes the location and name of the input PDF
           document to be appended.

             input_pdf_password{String}:
           A string that defines the master password to a protected file.  It
           must be a master password; a user password will not work."""
        if not self._open:
            raise Exception("File is closed for editing")
        self._pages += self._pagesForPath(pdf_path, input_pdf_password)

    def deletePages(self, page_range):
        """PDFDocument.deletePages(page_range)

           Provides the ability to delete one or multiple pages within an
           existing PDF document.

             page_range(String):
           A string that defines the page or pages to be deleted.  Delete a
           single page by passing in a single value as a string (for example,
           "3" ). Multiple pages can be deleted using a comma between each value
           (for example, "3, 5, 7" ).  Ranges can also be applied (for example,
           "1, 3, 5-12" )."""
        if not self._open:
            raise Exception("File is closed for editing")
        for page in reversed(sorted(set(page_range_list(page_range)))):
            if page > len(self._pages):
                raise IndexError("Page {} out of range".format(page))
            self._pages.pop(page - 1)

    def insertPages(self, pdf_path, before_page_number=1,
                    input_pdf_password=None):
        """PDFDocument.insertPages(pdf_path, {before_page_number},
           {input_pdf_password})

           Allows inserting the contents of one PDF document at the beginning or
           in between the pages of another  PDFDocument.

             pdf_path(String):
           A string that includes the location and name of the input PDF
           document to be inserted.

             before_page_number{Integer}:
           An integer that defines a page number in the currently referenced
           PDFDocument before which the new pages will be inserted.  For
           example, if the before_page_value is 1, the inserted page will be
           inserted before all pages.

             input_pdf_password{String}:
           A string that defines the master password to a protected file.   It
           must be a master password; a user password will not work."""
        if not self._open:
            raise Exception("File is closed for editing")
        if 0 >= before_page_number > len(self._pages):
            raise IndexError("Page {} out of range".format(before_page_number))
        pages_to_insert = self._pagesForPath(pdf_path, input_pdf_password)
        self._pages = (self._pages[:before_page_number - 1] +
                       pages_to_insert +
                       self._pages[before_page_number - 1:])

    def _writeToFile(self, pdf_path):
        import PyPDF2

        writer = PyPDF2.PdfFileWriter()
        writer.addMetadata(self._info)
        writer.setPageLayout(self._layout)
        writer.setPageMode(self._mode)
        if not self._pages:
            raise RuntimeError("PDF Document is empty")
        for page in self._pages:
            writer.addPage(page)
        # Copy over original document's password(s) if they were there
        if self._decrypt is not None:
            writer._encrypt = writer._addObject(self._decrypt)
            writer._encrypt_key = self._decrypt_key
            writer._ID = self._ID
        if self._password or self._owner_password:
            writer.encrypt(self._password if self._password is not None
                                          else self._owner_password,
                           self._owner_password,
                           self._128bit)
        with open(pdf_path, 'wb') as stream:
            writer.write(stream)

    def saveAndClose(self):
        """PDFDocument.saveAndClose()

           Saves any changes made to the currently referenced PDFDocument ."""
        if not self._open:
            raise Exception("File is closed for editing")
        self._open = False
        self._writeToFile(self._PDFPath)

    def updateDocProperties(self, pdf_title=None, pdf_author=None,
                            pdf_subject=None, pdf_keywords=None,
                            pdf_open_view=None, pdf_layout=None):
        """PDFDocument.updateDocProperties({pdf_title}, {pdf_author},
           {pdf_subject}, {pdf_keywords}, {pdf_open_view}, {pdf_layout})

           Allows updating of the PDF document metadata and can also set the
           certain behaviors that will trigger when the document is opened in
           Adobe Reader or Adobe Acrobat, such as the initial view mode and the
           page thumbnails view.

             pdf_title{String}:
           A string defining the document title, a PDF metadata property.

             pdf_author{String}:
           A string defining the document author, a PDF metadata property.

             pdf_subject{String}:
           A string defining the document subject, a PDF metadata property.

             pdf_keywords{String}:
           A string defining the document keywords, a PDF metadata property.

             pdf_open_view{String}:
           A string or number that will define the behavior to trigger when the
           PDF file is viewed. The default value is USETHUMBS, which will show
           the Adobe Reader Pages panel automatically when the PDF is opened.

            * VIEWER_DEFAULT:   Uses the application user preference when
            opening the file

            * USE_NONE:   Displays the document only; does not show other panels

            * USE_THUMBS:   Displays the document plus the Pages panel

            * USE_BOOKMARKS:   Displays the document plus the Bookmarks panel

            * FULL_SCREEN:   Displays the document  in full-screen viewing mode

            * LAYERS:   Displays the document plus the layers panel

            * ATTACHMENT:   Displays the document plus the attachment panel

             pdf_layout{String}:
           A string or number that will define the initial view mode to trigger
           when the PDF file is viewed.

            * DONT_CARE:   Uses the application user preference when opening the
            file

            * SINGLE_PAGE:   Uses single-page mode

            * ONE_COLUMN:   Uses one-column continuous mode

            * TWO_COLUMN_LEFT:   Uses two-column continuous mode with first page
            on left

            * TWO_COLUMN_RIGHT:   Uses two-column continuous mode with first
            page on right

            * TWO_PAGE_LEFT:   Uses two-page mode left

            * TWO_PAGE_RIGHT:   Uses two-page mode right"""
        import PyPDF2

        if not self._open:
            raise Exception("File is closed for editing")
        if pdf_title is not None:
            self._info[PyPDF2.generic.TextStringObject('/Title')] = \
                    PyPDF2.generic.TextStringObject(pdf_title)
        if pdf_author is not None:
            self._info[PyPDF2.generic.TextStringObject('/Author')] = \
                    PyPDF2.generic.TextStringObject(pdf_author)
        if pdf_subject is not None:
            self._info[PyPDF2.generic.TextStringObject('/Subject')] = \
                    PyPDF2.generic.TextStringObject(pdf_subject)
        if pdf_keywords is not None:
            self._info[PyPDF2.generic.TextStringObject('/Keywords')] = \
                    PyPDF2.generic.TextStringObject(pdf_keywords)
        open_view_map = {
                            'VIEWER_DEFAULT': '/UseNone',
                            'USE_NONE': '/UseNone',
                            'USE_THUMBS': '/UseThumbs',
                            'USE_BOOKMARKS': '/UseOutlines',
                            'FULL_SCREEN': '/FullScreen',
                            'LAYERS': '/UseOC',
                            'ATTACHMENT': '/UseAttachments'
                        }
        if pdf_open_view is not None:
            try:
                self._mode = open_view_map[pdf_open_view.upper()]
                if self._mode == '/FullScreen' and pdf_layout is None:
                    pdf_layout = 'SINGLE_PAGE'
            except KeyError:
                raise ValueError(pdf_open_view)
        layout_view_map = {
                                'DONT_CARE': '/NoLayout',
                                'SINGLE_PAGE': '/SinglePage',
                                'ONE_COLUMN': '/OneColumn',
                                'TWO_COLUMN_LEFT': '/TwoColumnLeft',
                                'TWO_COLUMN_RIGHT': '/TwoColumnRight',
                                'TWO_PAGE_LEFT': '/TwoPageLeft',
                                'TWO_PAGE_RIGHT': '/TwoPageRight'
                          }
        if pdf_layout is not None:
            try:
                self._layout = layout_view_map[pdf_layout.upper()]
            except KeyError:
                raise ValueError(pdf_layout)

    def updateDocSecurity(self, new_master_password, new_user_password=None,
                          encryption='RC4'):
        """PDFDocument.updateDocSecurity(new_master_password,
           {new_user_password}, {encryption})

           Provides the mechanism that sets password, encryption, and security
           restrictions on PDF files.

             new_master_password(String):
           A string that defines the master document password. This password is
           required for appending and inserting pages into a secured PDF.

             new_user_password{String}:
           A string that defines the user password needed to open the PDF
           document for viewing.

             encryption{String}:
           A string  that defines the encryption technique used on the PDF.
           This is the only encryption type supported.

            * "RC4":  Uses 128-bit RC4 encryption (Acrobat 5.0 compatible)"""
        if not self._open:
            raise Exception("File is closed for editing")
        self._password = new_master_password
        self._owner_password = new_user_password

    @property
    def pageCount(self):
        return len(self._pages)

def PDFDocumentOpen(pdf_path, user_password=None, master_password=None):
    """PDFDocumentOpen(pdf_path, {user_password}, {master_password})

       Returns a reference to  an existing PDFDocument file on disk.

         pdf_path(String):
       A string that specifies the path and file name of the PDF file to open.

         user_password{String}:
       A string that specifies the user password. The user password, if set, is
       what you need to provide in order to open a PDF.

         master_password{String}:
       A string that specifies the master password. The master password, if set,
       controls permissions, such as printing, editing, extracting, commenting,
       and so on."""
    return PDFDocument(pdf_path, user_password
                                 if master_password is None
                                 else master_password, False)

def PDFDocumentCreate(pdf_path):
    """PDFDocumentCreate(pdf_path)

       Creates a new PDFDocument object in memory.

         pdf_path(String):
       A string that specifies the  path and file name for the resulting PDF
       file when the saveAndClose method is called."""
    return PDFDocument(pdf_path, open_new=True)
