# -*- coding: utf-8 -*-
#COPYRIGHT 2018 ESRI
#
#TRADE SECRETS: ESRI PROPRIETARY AND CONFIDENTIAL
#Unpublished material - all rights reserved under the
#Copyright Laws of the United States.
#
#For additional information, contact:
#Environmental Systems Research Institute, Inc.
#Attn: Contracts Dept
#380 New York Street
#Redlands, California, USA 92373
#
#email: contracts@esri.com
r"""The ArcGIS Data Reviewer extension adds a data quality control
framework to ArcGIS. Data Reviewer validates data using different
checks, contained in batch jobs, that test spatial relationships and
integrity. It also provides tools to correct errors found during
validation."""
__all__ = ['CreateReviewerSession', 'DeleteReviewerSession',
           'EnableDataReviewer', 'ExecuteReviewerBatchJob',
           'WriteToReviewerTable']
__alias__ = 'Reviewer'
from arcpy.geoprocessing._base import gptooldoc, gp, gp_fixargs
from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject

# Data Validation toolset
@gptooldoc('ExecuteReviewerBatchJob_Reviewer', None)
def ExecuteReviewerBatchJob(reviewer_workspace=None, session=None, batch_job_file=None, production_workspace=None, analysis_area=None, changed_features=None, production_workspaceversion=None):
    """ExecuteReviewerBatchJob_Reviewer(reviewer_workspace, session, batch_job_file, {production_workspace}, {analysis_area}, {changed_features}, {production_workspaceversion})

        Runs a Reviewer batch job on a workspace and writes the results to a
        Reviewer session. A Reviewer batch job contains groups of Reviewer
        checks. Checks validate data based on conditions, rules, and spatial
        relationships. Checks also specify sets of features or rows to
        validate and their source workspace. A Reviewer session stores
        information about validation tasks performed by Reviewer checks. This
        information is stored in a table and a dataset in the Reviewer
        workspace.

     INPUTS:
      reviewer_workspace (Workspace):
          The workspace where the Reviewer batch job results are written.
      session (String):
          The identifier and name for a Reviewer session. The session must
          exist in the Reviewer workspace.
      batch_job_file (File):
          The path to the Reviewer batch job file to be executed.
      production_workspace {Workspace}:
          The production database on which you want to run the Reviewer batch
          job. If this parameter points to an enterprise geodatabase, verify
          that the connection file (.sde) references the correct geodatabase
          version.
      analysis_area {Feature Layer / Extent}:
          Polygon features or an arcpy.Extent object to define the area that
          will be used to build a validation processing area.
      changed_features {Boolean}:
          Indicates the type of features, changed or unchanged, to validate when
          the production workspace references data in an enterprise geodatabase.

          * CHANGED_FEATURES-Validates only features that changed from the
          parent to the child version.

          * ALL_FEATURES-Validates all features in the data referenced by the
          batch job. This is the default.
      production_workspaceversion {String}:
          The version of the production workspace to be validated by the batch
          job. This is only applicable when the production workspace is an
          enterprise geodatabase."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.ExecuteReviewerBatchJob_Reviewer(*gp_fixargs((reviewer_workspace, session, batch_job_file, production_workspace, analysis_area, changed_features, production_workspaceversion), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('WriteToReviewerTable_Reviewer', None)
def WriteToReviewerTable(in_reviewer_workspace=None, in_session=None, in_features=None, in_field=None, in_origin_table_name=None, in_review_status=None, in_subtype=None, in_notes=None, in_severity=None, in_check_title=None):
    """WriteToReviewerTable_Reviewer(in_reviewer_workspace, in_session, in_features, in_field, in_origin_table_name, in_review_status, {in_subtype}, {in_notes}, {in_severity}, {in_check_title})

        Writes a feature class, feature layer, table, or table view to the
        Reviewer workspace.

     INPUTS:
      in_reviewer_workspace (Workspace):
          The path to the Reviewer workspace where the features or table
          records will be written.
      in_session (String):
          The Reviewer session ID in which the features or table records will
          be written. Use the full session ID format: Session 1 : Session 1.
      in_features (Feature Layer / Table View):
          The features or table records to write to the Reviewer workspace.
      in_field (Field):
          The field that contains identifiers for the features. The value from
          this field populates the ID field in the Reviewer Results pane. The
          field you choose must have a data type of Long.
      in_origin_table_name (String / Field):
          The string or field value that will be used to populate the Source
          field in the Reviewer Results pane for each record that is written.
          This is typically the name of the feature class or table.

          * String-The name of the feature layer is defined as a text string.

          * Field-The value for the feature layer name is derived from a field
          on the feature layer or table.
      in_review_status (String / Field):
          A status string to associate with the group of records written to the
          Reviewer workspace. The default value is Write GP Results to Reviewer
          Table.If the default value is deleted or left blank, the value Write
          GP Results to Reviewer Table will be used as the status string.

          * String-You can type the value in the String text box.

          * Field-You can choose the Review Status field from the feature layer.
      in_subtype {String / Field}:
          The feature class subtype to which the features belong. This can be
          derived from a specified value or a field on the feature class. The
          value from this parameter populates the SUBTYPE field in the Reviewer
          Results pane.

          * String-You can type the value in the String text box.

          * Field-You can choose the Subtype value from a field on the feature
          layer.
      in_notes {String / Field}:
          Text that populates the Notes field in the Reviewer table. The notes
          are used to provide a more specific description of the feature or
          table record.

          * String-You can type the value in the String text box.

          * Field-You can choose the Notes value from a field on the feature
          layer.
      in_severity {String / Field}:
          A numeric value that represents the significance of the features or
          table records that have been written to the Reviewer workspace. The
          values range from 5 (low importance) to 1 (high priority). This value
          populates the Severity field in the Reviewer Results pane.

          * String-You can type the value in the String text box.

          * Field-You can choose the Severity value from a field on the feature
          layer.
      in_check_title {String / Field}:
          Text that populates the Check Title field found in the Reviewer
          Results pane. Check Title is used to describe the error condition
          detected on the feature or table record.

          * String-You can type the value in the String text box.

          * Field-You can choose the Check Title value from a field on the
          feature layer."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.WriteToReviewerTable_Reviewer(*gp_fixargs((in_reviewer_workspace, in_session, in_features, in_field, in_origin_table_name, in_review_status, in_subtype, in_notes, in_severity, in_check_title), True)))
        return retval
    except Exception as e:
        raise e


# Result Management toolset
@gptooldoc('CreateReviewerSession_Reviewer', None)
def CreateReviewerSession(reviewer_workspace=None, session_name=None, session_template=None, duplicate_checking=None, store_geometry=None, username=None, version=None):
    """CreateReviewerSession_Reviewer(reviewer_workspace, session_name, {session_template}, {duplicate_checking}, {store_geometry}, {username}, {version})

        Creates a new Reviewer session in the specified workspace.

     INPUTS:
      reviewer_workspace (Workspace):
          The workspace in which a new Reviewer session will be created.
      session_name (String):
          The name of the session that will be created in the Reviewer
          workspace.
      session_template {String}:
          An existing Reviewer session whose properties will be copied to the
          new session.
      duplicate_checking {String}:
          Determines how duplicate validation results are handled in the
          session.

          * NONE-Does not search for duplicate validation results. This will
          improve performance when writing validation results to the database.
          This is the default.

          * SESSION-Searches within the session for duplicate validation
          results.

          * DATABASE-Searches the entire database for duplicate validation
          results.
      store_geometry {Boolean}:
          Determines if a result includes an associated geometry.

          * STORE_GEOMETRY-Results will include both geometry and attribute
          information. This is the default.

          * DO_NOT_STORE_GEOMETRY-Results will include only attribute
          information. This can improve performance when writing validation
          results to the geodatabase.
      username {String}:
          The user name of the person creating this Reviewer session. This
          defaults to the currently logged in Windows user.
      version {String}:
          The enterprise geodatabase version to which the session will be
          associated. Only enabled when you choose a Reviewer workspace stored
          in an enterprise geodatabase."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.CreateReviewerSession_Reviewer(*gp_fixargs((reviewer_workspace, session_name, session_template, duplicate_checking, store_geometry, username, version), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('DeleteReviewerSession_Reviewer', None)
def DeleteReviewerSession(reviewer_workspace=None, session=None):
    """DeleteReviewerSession_Reviewer(reviewer_workspace, session;session...)

        Permanently deletes one or more sessions and all related records from
        a Reviewer workspace. A Reviewer session is used to store and manage
        the results of your
        data analysis throughout the life cycle of an anomaly. It allows you
        to organize the results discovered from running batch jobs and visual
        inspections.

     INPUTS:
      reviewer_workspace (Workspace):
          The workspace from which the Reviewer session will be deleted.
      session (String):
          The Reviewer session identifier and name. The session must exist in
          the Reviewer workspace, for example, Session 1 : data_qc."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.DeleteReviewerSession_Reviewer(*gp_fixargs((reviewer_workspace, session), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('EnableDataReviewer_Reviewer', None)
def EnableDataReviewer(workspace=None, spatial_reference=None, register_as_versioned=None, config_keyword=None):
    """EnableDataReviewer_Reviewer(workspace, {spatial_reference}, {register_as_versioned}, {config_keyword})

        Adds a feature dataset and tables necessary for an existing
        geodatabase to be considered a Reviewer workspace and store Data
        Reviewer results. The Reviewer workspace tables are required by ArcGIS
        Data Reviewer to manage Reviewer sessions.

     INPUTS:
      workspace (Workspace):
          The geodatabase where the Data Reviewer tables and feature dataset
          will be created. This can be a desktop or enterprise geodatabase.
      spatial_reference {Spatial Reference}:
          The geographic or projected coordinate system of the feature classes
          in the Reviewer workspace. The default is GCS_WGS_1984 if no value is
          specified.
      register_as_versioned {Boolean}:
          Indicates whether the feature classes and tables added to the
          workspace will be registered as versioned. This only applies to
          enterprise databases.

          * NONVERSIONED-The feature classes and tables will not be registered
          as versioned after they are added to the geodatabase. This is the
          default.

          * VERSIONED-The feature classes and tables will be registered as
          versioned after they are added to the geodatabase.
      config_keyword {String}:
          The configuration keyword that determines the storage parameters of
          the database tables. This applies to file and enterprise geodatabases.
          The DEFAULTS keyword is used by default."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.EnableDataReviewer_Reviewer(*gp_fixargs((workspace, spatial_reference, register_as_versioned, config_keyword), True)))
        return retval
    except Exception as e:
        raise e


# End of generated toolbox code
del gptooldoc, gp, gp_fixargs, convertArcObjectToPythonObject